@@ -26,6 +26,7 @@
  * demonstrates how to handle NV-CONTROL events.
  */
 
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -682,7 +683,7 @@ static AttrEntry attr_table[] = {
     MAKE_ENTRY(NV_CTRL_GVI_TEST_MODE),
     MAKE_ENTRY(NV_CTRL_COLOR_SPACE),
     MAKE_ENTRY(NV_CTRL_COLOR_RANGE),
-    MAKE_ENTRY(NV_CTRL_SHOW_GSYNC_VISUAL_INDICATOR),
+    MAKE_ENTRY(NV_CTRL_SHOW_VRR_VISUAL_INDICATOR),
     MAKE_ENTRY(NV_CTRL_GPU_SCALING_DEFAULT_TARGET),
     MAKE_ENTRY(NV_CTRL_GPU_SCALING_DEFAULT_METHOD),
     MAKE_ENTRY(NV_CTRL_DITHERING_MODE),
@@ -735,6 +736,8 @@ static AttrEntry attr_table[] = {
     MAKE_ENTRY(NV_CTRL_XV_SYNC_TO_DISPLAY_ID),
     MAKE_ENTRY(NV_CTRL_CURRENT_XV_SYNC_TO_DISPLAY_ID),
     MAKE_ENTRY(NV_CTRL_PALETTE_UPDATE_EVENT),
-    MAKE_ENTRY(NV_CTRL_GSYNC_ALLOWED),
+    MAKE_ENTRY(NV_CTRL_VRR_ALLOWED),
+    MAKE_ENTRY(NV_CTRL_DISPLAY_VRR_MODE),
+    MAKE_ENTRY(NV_CTRL_DISPLAY_VRR_MIN_REFRESH_RATE),
     { -1, NULL, NULL }
 };
