@@ -39,6 +39,10 @@ G_BEGIN_DECLS
 #define CTK_MANAGE_GRID_LICENSE_GET_CLASS(obj) \
     (G_TYPE_INSTANCE_GET_CLASS ((obj), CTK_TYPE_MANAGE_GRID_LICENSE, CtkMANAGE_GRID_LICENSEClass))
 
+#define GRID_LICENSE_INFO_MAX_LENGTH        128
+#define GRID_MESSAGE_MAX_BUFFER_SIZE        512
+#define GRID_VIRTUAL_APPLICATIONS           "GRID Virtual Applications"
+
 typedef struct _CtkManageGridLicense       CtkManageGridLicense;
 typedef struct _CtkManageGridLicenseClass  CtkManageGridLicenseClass;
 
@@ -56,13 +60,17 @@ struct _CtkManageGridLicense
     GtkWidget* txt_server_address;
     GtkWidget* label_license_state;
     GtkWidget* btn_apply;
+    GtkWidget* radio_btn_vapp;
+    GtkWidget* radio_btn_qdws;
     GtkWidget* btn_cancel;
     GtkWidget* box_server_info;
 
     DbusData *dbusData;
+    CtrlTarget *target;
     gint license_edition_state;
-    gint feature_type;                      // Feature type from UI/gridd.conf.
-    int gridd_feature_type;                 // Feature type fetched from nvidia-gridd.
+    gint feature_type;                              // Feature type from UI/gridd.conf.
+    int gridd_feature_type;                         // Feature type fetched from nvidia-gridd.
+    char productName[GRID_LICENSE_INFO_MAX_LENGTH]; // GRID product name fetched from nvml.
 };
 
 /*
@@ -82,6 +90,8 @@ typedef enum
     NV_GRID_LICENSE_EXPIRED_VGPU,
     NV_GRID_LICENSE_EXPIRED_QDWS,
     NV_GRID_LICENSE_RESTART_REQUIRED,
+    NV_GRID_LICENSE_RESTART_REQUIRED_VAPP,
+    NV_GRID_UNLICENSED_REQUEST_DETAILS,     // Default case
 } licenseStatus;
 
 struct _CtkManageGridLicenseClass
